//Camera class implementation

#include "main.h"

//Camera class control functionality implementation

//Camera position
void Camera::PositionCamera(GLfloat positionX, GLfloat positionY, 
							GLfloat positionZ, GLfloat viewX, 
							GLfloat viewY, GLfloat viewZ, 
							GLfloat upVectorX, GLfloat upVectorY, GLfloat upVectorZ)
{
	//Se the camera position varliables
	Vector3 vPosition	= Vector3(positionX, positionY, positionZ);
	Vector3 vView		= Vector3(viewX, viewY, viewZ);
	Vector3 vUpVector	= Vector3(upVectorX, upVectorY, upVectorZ);

	m_vPosition = vPosition;					// Assign the position
	m_vView     = vView;						// Assign the view
	m_vUpVector = vUpVector;					// Assign the up vector
}

//Camera rotation
void Camera::RotateView(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
{
	Vector3 vNewView;

	//Get the view vector 
	Vector3 vView = m_vView - m_vPosition;		

	//Calculate the sine and cosine of the angle once
	GLfloat cosTheta = (GLfloat)cos(angle);
	GLfloat sinTheta = (GLfloat)sin(angle);

	//Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* vView.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z;

	//Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* vView.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z;

	//Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* vView.z;

	m_vView = m_vPosition + vNewView;
}

//Camera mouse control
void Camera::SetViewByMouse()
{
	POINT mousePos;								// Mouse pos struct for X and Y coord values
	int middleX = S_WIDTH  >> 1;				// Binary shift for half window width
	int middleY = S_HEIGHT >> 1;				// Binary shift for half window height
	float angleY = 0.0f;						// Up or down
	float angleZ = 0.0f;						// Left or right
	static float currentRotX = 0.0f;
	
	// Get the mouse X,Y position
	GetCursorPos(&mousePos);						
	
	// If cursor is still in the middle, don't update the screen
	if( (mousePos.x == middleX) && (mousePos.y == middleY) ) return;

	// Set the mouse position to the middle of window
	SetCursorPos(middleX, middleY);							

	// Get the direction the mouse moved in
	angleY = (float)( (middleX - mousePos.x) ) / 1000.0f;		
	angleZ = (float)( (middleY - mousePos.y) ) / 1000.0f;		

	//Keep track of current rotation
	currentRotX -= angleZ;  

	
	if(currentRotX > 1.0f)
	{
		currentRotX = 1.0f;
	}
	else if(currentRotX < -1.0f)
	{
		currentRotX = -1.0f;
	}
	else
	{
		Vector3 vAxis = vAxis.Cross(m_vView - m_vPosition, m_vUpVector);
		vAxis = vAxis.Normalize(vAxis);
		RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z);
		RotateView(angleY, 0, 1, 0);
	}
}

//Camera strafing
void Camera::StrafeCamera(GLfloat speed)
{
	m_vPosition.x += m_vStrafe.x * speed;
	m_vPosition.z += m_vStrafe.z * speed;

	m_vView.x += m_vStrafe.x * speed;
	m_vView.z += m_vStrafe.z * speed;
}


//Camera movement
void Camera::MoveCamera(GLfloat speed)
{
	Vector3 vVector = m_vView - m_vPosition;

	vVector = vVector.Normalize(vVector);
	
	//Increase the speed of movement to the cameras position and view
	m_vPosition.x += vVector.x * speed;	
	m_vPosition.z += vVector.z * speed;		 
	m_vView.x += vVector.x * speed;			
	m_vView.z += vVector.z * speed;			
}

//Camera user input handling

//TODO: Make this into seperate input class
void Camera::CheckForMovement()
{
	if(GetKeyState(VK_UP) & 0x80 || GetKeyState('W') & 0x80) 
	{				
		MoveCamera(C_SPEED);				
	}

	if(GetKeyState(VK_DOWN) & 0x80 || GetKeyState('S') & 0x80) 
	{			
		MoveCamera(-C_SPEED);				
	}

	if(GetKeyState(VK_LEFT) & 0x80 || GetKeyState('A') & 0x80) 
	{			
		StrafeCamera(-C_SPEED);
	}

	if(GetKeyState(VK_RIGHT) & 0x80 || GetKeyState('D') & 0x80)
	{			
		StrafeCamera(C_SPEED);
	}	

	if(GetKeyState('Q') & 0x80)
	{
		m_vPosition.y += 0.1;
	}

	if(GetKeyState('Z') & 0x80)
	{
		m_vPosition.y -= 0.1;
	}

	//Wireframe display
	if(GetKeyState('V') & 0x80)
	{			
		glPolygonMode(GL_FRONT, GL_LINE);
	}	

	//Normal smooth shaded filled polygon display
	if(GetKeyState('C') & 0x80)
	{			
		glPolygonMode(GL_FRONT, GL_FILL);
	}	

	//turn the fog off and on
	if(GetKeyState('F') & 0x80)
	{			
		glEnable(GL_FOG);
	}	
	if(GetKeyState('G') & 0x80)
	{			
		glDisable(GL_FOG);
	}	

}

//Update the camera view and strafe vector
void Camera::Update()
{
	Vector3 vCross = vCross.Cross(m_vView - m_vPosition, m_vUpVector);

	m_vStrafe = m_vStrafe.Normalize(vCross);

	SetViewByMouse();

	CheckForMovement();
}

//Function call to set up gluLookAt() with the user generated camera coordinates
void Camera::Look()
{
	gluLookAt(m_vPosition.x, m_vPosition.y, m_vPosition.z,	
			  m_vView.x,	 m_vView.y,     m_vView.z,	
			  m_vUpVector.x, m_vUpVector.y, m_vUpVector.z);
}