//Main scene rendering implementation

#include "main.h"

//Global OpenGL specific variables
GLuint	Textures[10];
Model * SceneModels [40];
GLuint heightmap = 0;

//Lighting global parameter variables
GLfloat Light_Ambient[] = {0.4f, 0.4f, 0.4f, 1.0f};
GLfloat Light_Diffuse[] = {0.6f, 0.4f, 0.4f, 1.0f};
GLfloat Light_Position[] = {0.0f, 1000.0f, -200.0f, 1.0f};

//Fog global parameter variables
GLfloat fog_density = 0.008f;
GLfloat fog_color[4] = {0.4f, 0.2f, 0.2f, 0.3f};

//terrain object instance
DEFAULT_TERRAIN test_terrain;

//Scene initialization implementation
void InitScene(HWND _hwnd)
{
	ghwnd = _hwnd;
	GetClientRect(ghwnd, &gRect);	//get rect into our handy global rect
	InitializeOpenGL(gRect.right, gRect.bottom);// initialise openGL

	//Load the textures
	CreateGLTexture("./textures/sky_back.tga", Textures[0], true);
	CreateGLTexture("./textures/sky_front.tga", Textures[1], true);
	CreateGLTexture("./textures/sky_down.tga", Textures[2], true);
	CreateGLTexture("./textures/sky_up.tga", Textures[3], true);
	CreateGLTexture("./textures/sky_left.tga", Textures[4], true);
	CreateGLTexture("./textures/sky_right.tga", Textures[5], true);
	CreateGLTexture("./textures/ground.tga", Textures[6], false);
	CreateGLTexture("./textures/water.tga", Textures[7], false);

	//Load the height map
	test_terrain.LoadRAWMap("./terrain/terrain.raw", 256);
	test_terrain.InitTerrain();

	//load and compile models display list
	LoadModels();

	//OpenGL settings
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.3f, 0.4f, 0.2f, 1.0f);				// Clearing Color
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	glEnable(GL_LIGHTING);								//Enalbe lighting
	glEnable(GL_TEXTURE_2D);							//Enable texturing
	glEnable(GL_CULL_FACE);								//Enable back face culling

	//lighting 1 setup
	glLightfv(GL_LIGHT1, GL_AMBIENT,  Light_Ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  Light_Diffuse);
	glEnable(GL_LIGHT1);

	//Setup the fog
	glFogi(GL_FOG_MODE, GL_EXP2); //fog mode
	glFogfv(GL_FOG_COLOR, fog_color); //fog color
	glFogf(GL_FOG_DENSITY, fog_density); //fog density
	glHint(GL_FOG_HINT, GL_LINEAR); //dont care much about good looking fog

	//Also, do any other setting ov variables here for your app if you wish. 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);// Linear Filtering
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);// Linear Filtering
}

void LoadModels()
{
	//supermarket
	SceneModels[0] = new MilkshapeModel();
	if ( SceneModels[0]->loadModelData( "./buildings/mart.ms3d" ) == false )
	{
		MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
	}
	SceneModels[0]->reloadTextures();

	//petrol station
	SceneModels[1] = new MilkshapeModel();
	if ( SceneModels[1]->loadModelData( "./buildings/petrol_station.ms3d" ) == false )
	{
		MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
	}
	SceneModels[1]->reloadTextures();

	//ruinded house 1
	SceneModels[2] = new MilkshapeModel();
	if ( SceneModels[2]->loadModelData( "./buildings/ruined_house_1.ms3d" ) == false )
	{
		MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
	}
	SceneModels[2]->reloadTextures();

	//ruinded house 2
	SceneModels[3] = new MilkshapeModel();
	if ( SceneModels[3]->loadModelData( "./buildings/ruined_house_2.ms3d" ) == false )
	{
		MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
	}
	SceneModels[3]->reloadTextures();

	//ruined house 3
	SceneModels[4] = new MilkshapeModel();
	if ( SceneModels[4]->loadModelData( "./buildings/ruined_house_3.ms3d" ) == false )
	{
		MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
	}
	SceneModels[4]->reloadTextures();

	//20 random treets
	for(int i = 5; i < 25; ++i)
	{
		SceneModels[i] = new MilkshapeModel();
		if ( SceneModels[i]->loadModelData( "./tree/tree.ms3d" ) == false )
		{
			MessageBox( NULL, "Couldn't load model data", "Error", MB_OK | MB_ICONERROR );
		}
		SceneModels[i]->reloadTextures();
	}
}


//TODO: Make all the model setup a display list
void DrawTown()
{
	//supermarket
	glPushMatrix();
	{
		//scale
		glScalef(0.2f, 0.2f, 0.2f);

		//translate
		glTranslatef(1000.0f, 113.0f, 200.0f);

		//draw
		SceneModels[0]->draw();
	}
	glPopMatrix();

	//petrol station
	glPushMatrix();
	{
		//scale
		glScalef(0.09f, 0.09f, 0.09f);

		//translate
		glTranslatef(500.0f, 275.0f, 200.0f);

		//draw
		SceneModels[1]->draw();
	}
	glPopMatrix();

	//ruined house 1
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(700.0f, 580.0f, 1500.0f);

		//draw
		SceneModels[2]->draw();
	}
	glPopMatrix();

	//ruined house 2
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(1250.0f, 640.0f, 1500.0f);

		//draw
		SceneModels[3]->draw();
	}
	glPopMatrix();

	//ruined house 3
	glPushMatrix();
	{
		//scale
		glScalef(0.03f, 0.03f, 0.03f);

		//translate
		glTranslatef(2000.0f, 1020.0f, 1500.0f);

		//draw
		SceneModels[4]->draw();
	}
	glPopMatrix();

	//////////////
	//20 trees 
	//////////////

	/////////////
	//1st set
	/////////////

	//1
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2100.0f, 790.0f, 1500.0f);

		//draw
		SceneModels[5]->draw();
	}
	glPopMatrix();

	//2
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2400.0f, 790.0f, 1500.0f);

		//draw
		SceneModels[6]->draw();
	}
	glPopMatrix();

	//3
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2950.0f, 790.0f, 1600.0f);

		//draw
		SceneModels[7]->draw();
	}
	glPopMatrix();

	//4
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2750.0f, 790.0f, 1750.0f);

		//draw
		SceneModels[8]->draw();
	}
	glPopMatrix();

	//5
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2650.0f, 790.0f, 1950.0f);

		//draw
		SceneModels[9]->draw();
	}
	glPopMatrix();

	//6
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(2850.0f, 790.0f, 1400.0f);

		//draw
		SceneModels[10]->draw();
	}
	glPopMatrix();

	/////////////
	//2nd set
	/////////////

	//7
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3850.0f, 790.0f, 2400.0f);

		//draw
		SceneModels[11]->draw();
	}
	glPopMatrix();

	//8
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3650.0f, 790.0f, 2250.0f);

		//draw
		SceneModels[12]->draw();
	}
	glPopMatrix();

	//9
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3500.0f, 790.0f, 2700.0f);

		//draw
		SceneModels[13]->draw();
	}
	glPopMatrix();

	//10
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3600.0f, 790.0f, 1700.0f);

		//draw
		SceneModels[14]->draw();
	}
	glPopMatrix();

	//11
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3500.0f, 790.0f, 2150.0f);

		//draw
		SceneModels[15]->draw();
	}
	glPopMatrix();

	//12
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3300.0f, 790.0f, 2750.0f);

		//draw
		SceneModels[16]->draw();
	}
	glPopMatrix();

	//13
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3200.0f, 790.0f, 2100.0f);

		//draw
		SceneModels[17]->draw();
	}
	glPopMatrix();

	//14
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(3000.0f, 790.0f, 1000.0f);

		//draw
		SceneModels[18]->draw();
	}
	glPopMatrix();

	////////////
	//3rd set
	////////////

	//15
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(4000.0f, 790.0f, 3000.0f);

		//draw
		SceneModels[19]->draw();
	}
	glPopMatrix();

	//16
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(4200.0f, 790.0f, 3200.0f);

		//draw
		SceneModels[20]->draw();
	}
	glPopMatrix();

	//17
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(4450.0f, 790.0f, 3350.0f);

		//draw
		SceneModels[21]->draw();
	}
	glPopMatrix();

	//18
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(4700.0f, 790.0f, 3450.0f);

		//draw
		SceneModels[22]->draw();
	}
	glPopMatrix();

	//19
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(4850.0f, 790.0f, 3800.0f);

		//draw
		SceneModels[23]->draw();
	}
	glPopMatrix();

	//20
	glPushMatrix();
	{
		//scale
		glScalef(0.04f, 0.04f, 0.04f);

		//translate
		glTranslatef(5000.0f, 790.0f, 3900.0f);

		//draw
		SceneModels[24]->draw();
	}
	glPopMatrix();
}

//Static scene lighting
void RenderStaticLight()
{
	//light 1
	glPushMatrix();
	{
		glLightfv(GL_LIGHT1, GL_POSITION, Light_Position);
	}
	glPopMatrix();
}

void DrawScene()
{
	//Clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity(); 

	//Call the camera 
	g_Camera.Look();

	RenderStaticLight();

	//draw the terrain
	glPushMatrix();
	{
		glTranslatef(0.0f, 0.0f, 0.0f);
		test_terrain.TerrainRender();
	}
	glPopMatrix();

	DrawTown();

	SwapBuffers(ghdc);// Swap the frame buffers.
}
