//Terrain class interface

#ifndef _TERRAIN_H_
#define _TERRAIN_H_

#include "main.h"

//display list defines
#define TERRAIN_LIST 1

//RAW file data and map height struct
struct HEIGHT_DATA
{
	unsigned char* m_Data;
	int m_Size_;
};

//Abstract terrain class
class TERRAIN
{
public:

	//Default class constructor
	TERRAIN(){}

	int m_Size; //map size, must be power of two

	///////////////////////////////////////
	//Abstract rendering member function
	///////////////////////////////////////
	virtual void TerrainRender() = NULL;

	/////////////////////////////////////
	//RAW map data handling functions
	/////////////////////////////////////
	bool LoadRAWMap(char * file_name, int _size);
	bool UnloadRAWMap();

	///////////////////////////////////
	//Terrain data handling routines
	///////////////////////////////////
	inline void SetHeightScale(float scale){m_heightScale = scale;}
	inline void SetHeightAtPoint(unsigned char height, int x, int z){m_heightData.m_Data[(z * m_Size) + x] = height;}
	inline unsigned char GetHeightAtPoint(int x, int z){return (m_heightData.m_Data[(z * m_Size) + x]);}
	inline float GetHeightScale(int x, int z){return ((m_heightData.m_Data[(z * m_Size) + x]) * m_heightScale);}

	//Default class destructor
	~TERRAIN(){}

protected:

	HEIGHT_DATA m_heightData; //map height data
	float m_heightScale; //height scaling variable
	Vector3 m_TerrainNormals[512 * 512];
	Vector3 m_WaterNormals[512 * 512];
};

//Defualt height map rendering method class
class DEFAULT_TERRAIN : public TERRAIN
{
public:

	//defualt constructor
	DEFAULT_TERRAIN(){}

	//pre-calculations
	void ComputeNormals(); 
	void ComputeVertexNormals();

	//per frame calculations
	//add water sinusidal oscilation calculation for the water mesh

	//skybox
	void CreateSkyBox();

	//water
	void CreateWater();

	//Display lists
	void CreateTerrainDL();

	//Main rendering method
	void InitTerrain(); //set everything up in here before rendering
	void TerrainRender();

	//defualt destructor
	~DEFAULT_TERRAIN(){}
};

#endif